%writehb		Write a sparse matrix in Harwell-Boeing format
%
%usage : writehb( A, file [,double] )
%
%A      = sparse matrix to write to file
%file   = filename
%double = 0 for single precision (default) , 1 for double
%
%Note : This code requires an executable m2hbo or m2hbodouble in the PATH 
%       which can convert a matrix in matlab coordinate format (either 8
%       or 16 digit format)  into Harwell-Boeing Format. This code will attempt
%       to create a temporary file in the /tmp directory.



%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% These programs were prepared by the Regents of the University of
%% California at Los Alamos National Laboratory (the University) under
%% contract W-7405-ENG-36 with the U.S. Department of Energy (DOE) and
%% under Contract KC-07-01-01 with the Department of Energy(DOE),
%% Office of Science, Mathematical, Information, and Computational Sciences,
%% Applied Mathematical Sciences Program.  All rights in these programs
%% are reserved by the DOE and the University.

%% Permission is granted to the public to copy and use this software
%% without charge, provided that this Notice and the statements of 
%% authorship are reproduced on all copies. Neither the U.S. government 
%% nor the University makes any warranty, express or implied, or assumes 
%% any liability or responsibility for the use of this software.

%% AMG code written by Menno Verbeek, in collaboration with Jane Cullum
%%  and Wayne Joubert.
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function writehb( A, file, double )

if nargin<3
  double=0;
end

if ~issparse(A)
  disp('Warning, matrix A is not sparse, writing in sparse format anyway')
  A=sparse(A);
end

tmpfile=['/tmp/tmp_writehb_',num2str(floor(rand*10000))];
bindir=strrep(which('readhb'),'readhb.m','');

if double
  eval(['save ',tmpfile,' A -ascii -double'])
  eval(['!',bindir,'m2hbodouble < ',tmpfile,' > ',file]);
else
  eval(['save ',tmpfile,' A -ascii'])
  eval(['!',bindir,'m2hbo < ',tmpfile,' > ',file]);
end
delete(tmpfile);

